--- tun.c.orig	Thu Feb 10 12:16:51 2005
+++ tun.c	Thu Feb 10 12:54:46 2005
@@ -47,6 +47,11 @@
 
 #include "memdbg.h"
 
+#if defined(__FreeBSD__)
+#include <sys/param.h>
+#include <sys/linker.h>
+#endif
+
 bool
 is_dev_type (const char *dev, const char *dev_type, const char *match_type)
 {
@@ -718,6 +723,9 @@
   char tunname[256];
   char dynamic_name[256];
   bool dynamic_opened = false;
+#if defined(__FreeBSD__)
+  bool kldtried = false;
+#endif
 
   ipv6_support (ipv6, ipv6_explicitly_supported, tt);
 
@@ -754,6 +762,27 @@
 		    {
 		      dynamic_opened = true;
 		      break;
+		    } else {
+#if defined(__FreeBSD__)
+		      if (errno == ENXIO || errno == ENOENT)
+		        {
+			  if (i == 0 && kldtried == false)
+			    {
+			      msg (M_INFO, "Trying to load tun/tap KLD");
+			      i--;
+			      kldtried = true;
+			      if (!strncmp("tun", dev, 3))
+			        if (kldload("if_tun.ko") == -1)
+			          msg (M_WARN | M_ERRNO, "Unable to load if_tun.ko");
+			      else if (!strncmp("tap", dev, 3))
+			        if (kldload("if_tap.ko") == -1)
+			          msg (M_WARN | M_ERRNO, "Unable to load if_tap.ko");
+			      else
+			        msg (M_FATAL, "Cannot determine which KLD to load from device name");
+                            }
+		        }
+		      continue;
+#endif
 		    }
 		  msg (D_READ_WRITE | M_ERRNO, "Tried opening %s (failed)", tunname);
 		}
